@@ -120,7 +120,7 @@ class _IMAPExtension(object):
                     log.warn(self.format_error('[%s UID=%s] Could not peek at message header' % (format_tags(tags), messageUID), error))
                     continue
                 # Yield
-                yield Email(self, messageUID, tags, data[0][1])
+                yield Email(self, messageUID, folder, data[0][1])
 
     def revive(self, targetFolder, message):
         'Upload the message to the targetFolder of the mail server'
@@ -133,8 +133,9 @@ class _IMAPExtension(object):
         else:
             self.create(targetFolder)
             folder = targetFolder
-        # Append
-        r, data = self.append(folder, '', mktime_tz(parsedate_tz(message['Date'])), message.as_string(False))
+        # A message without a date will return None instead of raising KeyError
+        messageDate = message['date']
+        r, data = self.append(folder, '', mktime_tz(parsedate_tz(messageDate)) if messageDate else None, message.as_string(False))
         if r != 'OK':
             raise IMAPError(self.format_error('Could not revive message', data))
         return data[0]
@@ -204,25 +205,24 @@ class IMAPError(Exception):
 class Email(object):
     'Convenience class representing an email from an IMAP mailbox'
 
-    def __init__(self, server, uid, tags, header):
+    def __init__(self, server, uid, folder, header):
         self.server = server
         self.uid = uid
-        self.tags = tags
+        self.folder = folder
+        self.tags = parse_tags(folder)
         # Parse header
         valueByKey = HeaderParser().parsestr(header)
         def getWhom(field):
             return ', '.join(formataddr((self._decode(x), y)) for x, y in getaddresses(valueByKey.get_all(field, [])))
         # Extract fields
-        if 'Date' in valueByKey:
-            timePack = parsedate_tz(valueByKey['Date'])
-            self.whenUTC = datetime.datetime.utcfromtimestamp(timegm(timePack) if timePack[-1] is None else mktime_tz(timePack)) if timePack else None
-        else:
-            self.whenUTC = None
-        self.subject = self._decode(valueByKey.get('Subject', ''))
-        self.fromWhom = getWhom('From')
-        self.toWhom = getWhom('To')
-        self.ccWhom = getWhom('CC')
-        self.bccWhom = getWhom('BCC')
+        self.date = valueByKey.get('date')
+        timePack = parsedate_tz(self.date)
+        self.whenUTC = datetime.datetime.utcfromtimestamp(timegm(timePack) if timePack[-1] is None else mktime_tz(timePack)) if timePack else None
+        self.subject = self._decode(valueByKey.get('subject', ''))
+        self.fromWhom = getWhom('from')
+        self.toWhom = getWhom('to')
+        self.ccWhom = getWhom('cc')
+        self.bccWhom = getWhom('bcc')
 
     def format_error(self, text, data):
         'Format an error that happened with a message'
@@ -241,9 +241,6 @@ class Email(object):
         string = ''.join(part.decode(encoding or 'utf-8', 'ignore') for part, encoding in packs)
         return pattern_whitespace.sub(' ', string.strip())
 
-    def format_error(self, text, data=None):
-        'Format error string'
-
     @property
     def flags(self):
         'Get flags'
@@ -258,6 +255,13 @@ class Email(object):
         'Set flags'
         if not hasattr(flags, '__iter__'):
             flags = [flags]
+        else:
+            flags = list(flags)
+        try:
+            # Remove flags that we cannot set
+            flags.remove(r'\Recent')
+        except ValueError:
+            pass
         r, data = self.server.uid('store', self.uid, 'FLAGS', '(%s)' % ' '.join(flags))
         if r != 'OK':
             raise IMAPError(self.format_error('Could not set flags', data))
@@ -290,12 +294,8 @@ class Email(object):
         'Flag the email as deleted or not'
         return self.set_flag(r'\Deleted', on)
 
-    def save(self, targetPath=None):
-        """
-        Save email to the hard drive and return a list of parts by index, type, name.
-        Compress the file if the filename ends with .gz
-        Return partPacks if targetPath=None.
-        """
+    def as_string(self, unixfrom=False):
+        'Fetch mime string from server'
         try:
             # Get
             flags = self.flags
@@ -303,11 +303,23 @@ class Email(object):
             r, data = self.server.uid('fetch', self.uid, '(RFC822)')
             if r != 'OK':
                 raise IMAPError(self.format_error('Could not fetch message body', data))
-            message = email.message_from_string(data[0][1])
             # Restore
             self.flags = flags
         except imaplib.IMAP4.abort, error:
             raise IMAPError(self.format_error('Connection failed while fetching message body', error))
+        return data[0][1]
+
+    def as_message(self):
+        'Fetch mime string from server and convert to email.message.Message'
+        return email.message_from_string(self.as_string())
+
+    def save(self, targetPath=None):
+        """
+        Save email to the hard drive and return a list of parts by index, type, name.
+        Compress the file if the filename ends with .gz
+        Return partPacks if targetPath=None.
+        """
+        message = self.as_message()
         # Save
         if targetPath:
             Generator((gzip.open if targetPath.endswith('.gz') else open)(targetPath, 'wb')).flatten(message)
@@ -392,12 +404,12 @@ def build_message(whenUTC=None, subject='', fromWhom='', toWhom='', ccWhom='', b
         message = mimeHTML
     else:
         message = mimeText
-    message['Date'] = formatdate(timegm((whenUTC or datetime.datetime.utcnow()).timetuple()))
-    message['Subject'] = subject.encode('utf-8')
-    message['From'] = fromWhom.encode('utf-8')
-    message['To'] = toWhom.encode('utf-8')
-    message['CC'] = ccWhom.encode('utf-8')
-    message['BCC'] = bccWhom.encode('utf-8')
+    message['date'] = formatdate(timegm((whenUTC or datetime.datetime.utcnow()).timetuple()))
+    message['subject'] = subject.encode('utf-8')
+    message['from'] = fromWhom.encode('utf-8')
+    message['to'] = toWhom.encode('utf-8')
+    message['cc'] = ccWhom.encode('utf-8')
+    message['bcc'] = bccWhom.encode('utf-8')
     return message
 
 
